@@ -78,6 +78,56 @@ static void cc_neg(point_ptr r, point_ptr p)
     element_neg(r->y, r->y);
 }
 
+static inline void cc_double_no_check(point_ptr r, point_ptr p)
+{
+    element_t lambda, e0, e1;
+    common_curve_ptr cc = p->curve->data;
+
+    element_init(lambda, p->curve->field);
+    element_init(e0, p->curve->field);
+    element_init(e1, p->curve->field);
+    //same point: double them
+
+    //lambda = (3x^2 + a) / 2y
+    element_set_si(e0, 3);
+    element_mul(lambda, p->x, p->x);
+    element_mul(lambda, lambda, e0);
+    element_add(lambda, lambda, cc->a);
+    element_add(e0, p->y, p->y);
+    element_invert(e0, e0);
+    element_mul(lambda, lambda, e0);
+    //x1 = lambda^2 - 2x
+    element_add(e1, p->x, p->x);
+    element_mul(e0, lambda, lambda);
+    element_sub(e0, e0, e1);
+    //y1 = (x - x1)lambda - y
+    element_sub(e1, p->x, e0);
+    element_mul(e1, e1, lambda);
+    element_sub(e1, e1, p->y);
+
+    element_set(r->x, e0);
+    element_set(r->y, e1);
+    r->inf_flag = 0;
+
+    element_clear(lambda);
+    element_clear(e0);
+    element_clear(e1);
+    return;
+}
+
+static void cc_double(point_ptr r, point_ptr p)
+{
+    if (point_is_inf(p)) {
+	point_set_inf(r);
+	return;
+    }
+    if (element_is0(p->y)) {
+	point_set_inf(r);
+	return;
+    }
+    cc_double_no_check(r, p);
+}
+
 static void cc_add(point_ptr r, point_ptr p, point_ptr q)
 {
     if (point_is_inf(p)) {
@@ -94,38 +144,7 @@ static void cc_add(point_ptr r, point_ptr p, point_ptr q)
 		point_set_inf(r);
 		return;
 	    } else {
-		element_t lambda, e0, e1;
-		common_curve_ptr cc = p->curve->data;
-
-		element_init(lambda, p->curve->field);
-		element_init(e0, p->curve->field);
-		element_init(e1, p->curve->field);
-		//same point: double them
-
-		//lambda = (3x^2 + a) / 2y
-		element_set_si(e0, 3);
-		element_mul(lambda, p->x, p->x);
-		element_mul(lambda, lambda, e0);
-		element_add(lambda, lambda, cc->a);
-		element_add(e0, p->y, p->y);
-		element_invert(e0, e0);
-		element_mul(lambda, lambda, e0);
-		//x1 = lambda^2 - 2x
-		element_add(e1, p->x, p->x);
-		element_mul(e0, lambda, lambda);
-		element_sub(e0, e0, e1);
-		//y1 = (x - x1)lambda - y
-		element_sub(e1, p->x, e0);
-		element_mul(e1, e1, lambda);
-		element_sub(e1, e1, p->y);
-
-		element_set(r->x, e0);
-		element_set(r->y, e1);
-		r->inf_flag = 0;
-
-		element_clear(lambda);
-		element_clear(e0);
-		element_clear(e1);
+		cc_double_no_check(r, p);
 		return;
 	    }
 	}
@@ -171,7 +190,7 @@ static void cc_mul(point_ptr r, mpz_ptr n, point_ptr p)
     point_set_inf(result);
 
     if (mpz_sgn(n)) for (s = mpz_sizeinbase(n, 2) - 1; s>=0; s--) {
-	point_add(result, result, result);
+	point_double(result, result);
 	if (mpz_tstbit(n, s)) {
 	    point_add(result, result, p);
 	}
@@ -206,6 +225,7 @@ void curve_init_cc_ab(curve_ptr c, element_ptr a, element_ptr b)
     c->from_x = cc_from_x;
     c->from_hash = cc_from_hash;
     c->neg = cc_neg;
+    c->doublefn = cc_double;
     c->add = cc_add;
     c->mul = cc_mul;
     c->data = malloc(sizeof(common_curve_t));
@@ -262,6 +282,7 @@ void cc_switch_field(curve_ptr cnew, curve_ptr c,
     cnew->random = c->random;
     cnew->neg = c->neg;
     cnew->add = c->add;
+    cnew->doublefn = c->doublefn;
     cnew->mul = c->mul;
 
     cnew->data = malloc(sizeof(common_curve_t));
@@ -312,170 +333,6 @@ void point_map(point_t R, fieldmap map, point_t P)
     R->inf_flag = 0;
 }
 
-void cc_miller_vertical(element_t v, point_t Q, point_t Z)
-{
-    element_t t0;
-
-    if (point_is_inf(Z)) {
-	return;
-    }
-    element_init(t0, v->field);
-    element_sub(t0, Q->x, Z->x);
-    element_mul(v, v, t0);
-    element_clear(t0);
-
-    return;
-}
-
-void cc_miller_tangent(element_t v, point_t Q, point_t Z)
-{
-    //a = -slope_tangent(Z.x, Z.y);
-    //b = 1;
-    //c = -(Z.y + a * Z.x);
-    //but we multiply by 2*Z.y to avoid division
-
-    //a = -Zx * (Zx + Zx + Zx + *twicea2) - *a4;
-    //Common curves: a2 = 0
-    element_t a, b, c;
-    element_t t0;
-    common_curve_ptr cc = Z->curve->data;
-
-    if (point_is_inf(Z)) {
-	return;
-    }
-
-    if (element_is0(Z->y)) {
-	//order 2
-	cc_miller_vertical(v, Q, Z);
-	return;
-    }
-
-    element_init(a, v->field);
-    element_init(b, v->field);
-    element_init(c, v->field);
-    element_init(t0, v->field);
-
-    element_add(a, Z->x, Z->x);
-    element_add(a, a, Z->x);
-    element_mul(a, a, Z->x);
-    element_add(a, a, cc->a);
-    element_neg(a, a);
-
-    element_add(b, Z->y, Z->y);
-
-    element_mul(t0, b, Z->y);
-    element_mul(c, a, Z->x);
-    element_add(c, c, t0);
-    element_neg(c, c);
-
-    element_mul(a, a, Q->x);
-    element_mul(b, b, Q->y);
-    element_add(c, c, a);
-    element_add(c, c, b);
-    element_mul(v, v, c);
-
-    element_clear(a);
-    element_clear(b);
-    element_clear(c);
-    element_clear(t0);
-}
-
-void cc_miller_line(element_t v, point_t Q, point_t A, point_t B)
-{
-    //a = -(B.y - A.y) / (B.x - A.x);
-    //b = 1;
-    //c = -(A.y + a * A.x);
-    //but we'll multiply by B.x - A.x to avoid division
-    element_t a, b, c;
-    element_t t0;
-
-    //we assume B is never O
-    if (point_is_inf(A)) {
-	cc_miller_vertical(v, Q, A);
-	return;
-    }
-
-    if (!element_cmp(A->x, B->x)) {
-	if (!element_cmp(A->y, B->y)) {
-	    cc_miller_tangent(v, Q, A);
-	    return;
-	}
-	cc_miller_vertical(v, Q, A);
-	return;
-    }
-
-    element_init(a, v->field);
-    element_init(b, v->field);
-    element_init(c, v->field);
-    element_init(t0, v->field);
-
-    element_sub(b, B->x, A->x);
-    element_sub(a, A->y, B->y);
-    element_mul(t0, b, A->y);
-    element_mul(c, a, A->x);
-    element_add(c, c, t0);
-    element_neg(c, c);
-
-    element_mul(a, a, Q->x);
-    element_mul(b, b, Q->y);
-    element_add(c, c, a);
-    element_add(c, c, b);
-    element_mul(v, v, c);
-
-    element_clear(a);
-    element_clear(b);
-    element_clear(c);
-    element_clear(t0);
-}
-
-//assumes P is in the base field, Q in some field extension,
-//and will be used in Tate pairing computation
-void cc_miller(element_t res, mpz_t q, point_t Pbase, point_t Q,
-	fieldmap mapbase)
-//TODO: optimize using the fact that P lies in the base field
-//(we don't need Q until the last minute in miller_*()'s)
-{
-    //collate divisions
-    int m;
-    element_t v, vd;
-    point_t P;
-    point_t Z;
-
-    element_init(v, res->field);
-    element_init(vd, res->field);
-    point_init(Z, Q->curve);
-    point_init(P, Q->curve);
-
-    point_map(P, mapbase, Pbase);
-    point_set(Z, P);
-
-    element_set1(v);
-    element_set1(vd);
-    m = mpz_sizeinbase(q, 2) - 2;
-
-    while(m >= 0) {
-	element_mul(v, v, v);
-	element_mul(vd, vd, vd);
-	cc_miller_tangent(v, Q, Z);
-	point_add(Z, Z, Z);
-	cc_miller_vertical(vd, Q, Z);
-	if (mpz_tstbit(q, m)) {
-	    cc_miller_line(v, Q, Z, P);
-	    point_add(Z, Z, P);
-	    cc_miller_vertical(vd, Q, Z);
-	}
-	m--;
-    }
-
-    element_invert(vd, vd);
-    element_mul(res, v, vd);
-
-    element_clear(v);
-    element_clear(vd);
-    point_clear(Z);
-    point_clear(P);
-}
-
 void cc_param_init(cc_param_ptr param)
 {
     mpz_init(param->q);
@@ -587,3 +444,116 @@ void cc_param_inp_str(cc_param_ptr p, FILE *stream)
     mpz_clear(k);
     token_clear(tok);
 }
+
+static void curve_group_init(element_ptr e)
+{
+    e->data = malloc(sizeof(point_t));
+    point_init(e->data, ((curve_group_data_ptr) e->field->data)->curve);
+}
+
+static void curve_group_clear(element_ptr e)
+{
+    point_clear(e->data);
+    free(e->data);
+}
+
+static void curve_group_invert(element_ptr x, element_ptr a)
+{
+    point_neg(x->data, a->data);
+}
+
+static void curve_group_mul(element_ptr x, element_ptr a, element_ptr b)
+{
+    point_add(x->data, a->data, b->data);
+}
+
+static void curve_group_set1(element_ptr x)
+{
+    point_set_inf(x->data);
+}
+
+static void curve_group_set(element_ptr x, element_ptr y)
+{
+    point_set(x->data, y->data);
+}
+
+static void curve_group_random(element_ptr x)
+{
+    curve_group_data_ptr p = x->field->data;
+    point_random(x->data);
+    point_mul(x->data, p->cofac, x->data);
+}
+
+static void curve_group_from_hash(element_ptr x, int len, void *data)
+{
+    curve_group_data_ptr p = x->field->data;
+    point_from_hash(x->data, len, data);
+    point_mul(x->data, p->cofac, x->data);
+}
+
+static void curve_group_out_str(FILE *stream, int base, element_ptr x)
+{
+    point_out_str(stream, base, x->data);
+}
+
+static int curve_group_length_in_bytes(element_ptr x)
+{
+    point_ptr P = x->data;
+    return element_length_in_bytes(P->x) + element_length_in_bytes(P->y);
+}
+
+static int curve_group_to_bytes(unsigned char *data, element_t e)
+{
+    point_ptr P = e->data;
+    int len;
+    len = element_to_bytes(data, P->x);
+    len += element_to_bytes(data + len, P->y);
+    return len;
+}
+
+static int curve_group_from_bytes(element_t e, unsigned char *data)
+{
+    point_ptr P = e->data;
+    int len;
+
+    //TODO: use point_set() instead of messing with internal data structure
+    P->inf_flag = 0;
+    len = element_from_bytes(P->x, data);
+    len += element_from_bytes(P->y, data + len);
+    return len;
+}
+
+void field_clear_curve_group(field_t f)
+{
+    curve_group_data_ptr p;
+    p = f->data;
+    mpz_clear(p->cofac);
+    free(p);
+}
+
+void field_init_curve_group(field_t f, curve_t c, mpz_t cofac)
+{
+    curve_group_data_ptr p;
+    p = f->data = malloc(sizeof(curve_group_data_t));
+    p->curve = c;
+    mpz_init(p->cofac);
+    mpz_set(p->cofac, cofac);
+    f->init = curve_group_init;
+    f->clear = curve_group_clear;
+    f->invert = curve_group_invert;
+    f->mul = curve_group_mul;
+    f->set1 = curve_group_set1;
+    f->set = curve_group_set;
+    f->random = curve_group_random;
+    f->from_hash = curve_group_from_hash;
+    f->out_str = curve_group_out_str;
+    f->field_clear = field_clear_curve_group;
+    if (c->field->fixed_length_in_bytes < 0) {
+	f->length_in_bytes = curve_group_length_in_bytes;
+    } else {
+	f->fixed_length_in_bytes = 2 * c->field->fixed_length_in_bytes;
+    }
+    f->to_bytes = curve_group_to_bytes;
+    f->from_bytes = curve_group_from_bytes;
+}
+
