@@ -21,6 +21,7 @@ struct curve_s {
     void (*from_hash)(point_ptr, int, void *);
     void (*neg)(point_ptr, point_ptr);
     void (*add)(point_ptr, point_ptr, point_ptr);
+    void (*doublefn)(point_ptr, point_ptr);
     void (*mul)(point_ptr, mpz_ptr, point_ptr);
     void (*curve_clear)(struct curve_s *curve);
     void *data;
@@ -68,6 +69,11 @@ static inline void point_add(point_ptr r, point_ptr p, point_ptr q)
     r->curve->add(r, p, q);
 }
 
+static inline void point_double(point_ptr r, point_ptr p)
+{
+    r->curve->doublefn(r, p);
+}
+
 static inline void point_set(point_ptr p, point_ptr q)
 {
     if (q->inf_flag) {
@@ -110,6 +116,8 @@ static inline void point_from_x(point_ptr p, element_ptr x)
     p->curve->from_x(p, x);
 }
 
+void point_map(point_t R, fieldmap map, point_t P);
+
 //I define "common" curves (abbr. "cc") to be those
 //of the form y^2 = x^3 + ax + b defined over a field with
 //characteristic > 3
@@ -120,8 +128,6 @@ struct common_curve_s {
 typedef struct common_curve_s common_curve_t[1];
 typedef struct common_curve_s *common_curve_ptr;
 
-void cc_init_extend(curve_ptr cnew, curve_ptr c, field_ptr fext);
-
 struct cc_param_s {
     mpz_t q; //curve defined over F_q
     mpz_t n; //has order n (= q - t + 1) in F_q
@@ -141,4 +147,13 @@ void cc_param_clear(cc_param_ptr cc);
 void cc_param_set(cc_param_ptr x, cc_param_ptr y);
 void cc_param_out_str(FILE *stream, cc_param_ptr p);
 void cc_param_inp_str(cc_param_ptr p, FILE *stream);
+
+//for shoehorning the group of points on a curve into the `field' data type
+struct curve_group_data_s {
+    curve_ptr curve;
+    mpz_t cofac;
+};
+typedef struct curve_group_data_s curve_group_data_t[1];
+typedef struct curve_group_data_s *curve_group_data_ptr;
+
 #endif //CURVE_H
