@@ -1,4 +1,7 @@
-//TODO: use memmove for remove routines?
+#undef malloc
+#undef realloc
+#undef free
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
