@@ -23,4 +23,14 @@ void field_init_fi(field_ptr f, field_ptr fbase);
 //naturally map an element from a field K to K[i]
 void element_field_to_fieldi(element_ptr r, element_ptr a);
 
+static inline element_ptr fi_re(element_ptr a)
+{
+    return ((fi_data_ptr) a->data)->x;
+}
+
+static inline element_ptr fi_im(element_ptr a)
+{
+    return ((fi_data_ptr) a->data)->y;
+}
+
 #endif //FIELDI_H
