@@ -432,7 +432,7 @@ int findroot(element_ptr root, element_ptr poly)
     mpz_init(q);
     mpz_set(q, poly_base_field(poly)->order);
 
-    field_init_polymod_no_nqr(fpxmod, poly);
+    field_init_polymod(fpxmod, poly);
     element_init(p, fpxmod);
     element_init(x, fpxmod);
     poly_setx(x); 
@@ -467,7 +467,7 @@ int findroot(element_ptr root, element_ptr poly)
 	    //found a root!
 	    break;
 	}
-	field_init_polymod_no_nqr(fpxmod, p);
+	field_init_polymod(fpxmod, p);
 	element_init(r, fpxmod);
 step_random:
 	element_random(r);
