@@ -4,14 +4,6 @@
 #include "curve.h"
 #include "solinas.h"
 
-//for shoehorning the group of points on a curve into the `field' data type
-struct curve_group_data_s {
-    curve_ptr curve;
-    mpz_t cofac;
-};
-typedef struct curve_group_data_s curve_group_data_t[1];
-typedef struct curve_group_data_s *curve_group_data_ptr;
-
 struct pairing_s {
     mpz_t r; //order of G1, G2, GT
     field_t Zr; //the field Z_r
@@ -45,7 +37,8 @@ typedef struct mnt_pairing_data_s *mnt_pairing_data_ptr;
 struct solinas_pairing_data_s {
     field_t Fq, Fq2;
     curve_t Eq, Eq2;
-    mpz_t tateexp;
+    mpz_t tateexp; //TODO: get rid of?
+    mpz_t h;
     int exp2, exp1;
     int sign1, sign0;
 };
