@@ -10,21 +10,18 @@ void poly_alloc(element_ptr e, int n)
     element_ptr e0;
     int k;
     k = p->coeff->count;
-    if (k < n) {
-	while (k < n) {
-	    e0 = malloc(sizeof(element_t));
-	    element_init(e0, pdp->field);
-	    darray_append(p->coeff, e0);
-	    k++;
-	}
-    } else if (k > n) {
-	while (k > n) {
-	    k--;
-	    e0 = darray_at(p->coeff, k);
-	    element_clear(e0);
-	    free(e0);
-	    darray_remove_last(p->coeff);
-	}
+    while (k < n) {
+	e0 = malloc(sizeof(element_t));
+	element_init(e0, pdp->field);
+	darray_append(p->coeff, e0);
+	k++;
+    }
+    while (k > n) {
+	k--;
+	e0 = darray_at(p->coeff, k);
+	element_clear(e0);
+	free(e0);
+	darray_remove_last(p->coeff);
     }
 }
 
@@ -42,6 +39,7 @@ static void poly_clear(element_ptr e)
 
     poly_alloc(e, 0);
     darray_clear(p->coeff);
+    free(e->data);
 }
 
 void poly_remove_leading_zeroes(element_ptr e)
@@ -229,6 +227,7 @@ void poly_const_mul(element_ptr res, element_ptr a, element_ptr poly)
     }
     poly_remove_leading_zeroes(r);
     element_set(res, r);
+    element_clear(r);
 }
 
 void poly_random_monic(element_ptr f, int deg)
@@ -288,6 +287,7 @@ static void polymod_mul(element_ptr res, element_ptr f, element_ptr g)
 
     element_set(res, prod);
     element_clear(prod);
+    element_clear(p0);
 }
 
 static void poly_set0(element_ptr e)
@@ -486,7 +486,6 @@ void field_init_poly(field_ptr f, field_ptr base_field)
     f->neg = poly_neg;
     f->mul = poly_mul;
     f->cmp = poly_cmp;
-    mpz_init(f->order);
 }
 
 static void field_clear_polymod(field_ptr f)
@@ -494,6 +493,7 @@ static void field_clear_polymod(field_ptr f)
     polymod_field_data_ptr p = f->data;
     void clear(void *p) {
 	element_clear(p);
+	free(p);
     }
 
     darray_forall(p->xpwr, clear);
@@ -739,7 +739,6 @@ void field_init_polymod(field_ptr f, element_ptr poly)
     } else {
 	f->fixed_length_in_bytes = p->field->fixed_length_in_bytes * poly_degree(poly);
     }
-    mpz_init(f->order);
     n = pd->coeff->count;
     mpz_pow_ui(f->order, p->field->order, n - 1);
 
